import org.apache.el.lang.*;
import org.apache.el.parser.Node;
import org.apache.jasper.el.ELContextImpl;
import org.testng.*;
import org.testng.annotations.Test;

import javax.el.*;


public class BeanELResolverTest {


	@Test
	public void testMethodWithFixArgumentList() {
		BeanELResolver beanELResolver = new MyBeanELResolver();
		ELContextImpl elContext = new ELContextImpl(beanELResolver);

		String method1Result = (String) readExpressionValue("#{bean.method1('ab','cd')}", elContext);
		Assert.assertEquals(method1Result, "ab_method1_cd");

		String method2Result = (String) readExpressionValue("#{bean.method2('ab','cd', 'ef')}", elContext);
		Assert.assertEquals(method2Result, "ab_method2_cd_ef");

	}


	/**
	 * Reads the value specified by the given EL expression (using the given ELContext).
	 */
	static public Object readExpressionValue(String expression, ELContext elContext) {
		Node node = ExpressionBuilder.createNode(expression);
		Object resolvedExpression = node.getValue(new EvaluationContext(elContext, null, null));
		return resolvedExpression;
	}


	/**
	 * This class is only used to get access to an object of type {@link Bean} with an expression #{bean}.
	 * Usually, this is done with a different EL resolver registered in a {@link CompositeELResolver}, but for this bug
	 * reproducer this solution is the more simple one.
	 */
	static class MyBeanELResolver extends BeanELResolver {

		private Bean bean = new Bean();

		@Override
		public Object getValue(ELContext context, Object base, Object property) {
			if (base == null && "bean".equals(property.toString())) {
				context.setPropertyResolved(true);
				return bean;
			}
			else {
				return super.getValue(context, base, property);
			}
		}

	}


}



